/*
Copyright 2008 York University

Licensed under the Educational Community License (ECL), Version 2.0 or the New
BSD license. You may not use this file except in compliance with one these
Licenses.

You may obtain a copy of the ECL 2.0 License and BSD License at
https://source.fluidproject.org/svn/LICENSE.txt

 */

package org.fluidproject.vulab.rascal.core;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.TargetDataLine;

import org.fluidproject.vulab.rascal.ui.ScreencastApplet;
import org.fluidproject.vulab.rascal.ui.Settings;
import org.fluidproject.vulab.rascal.utils.http.HTTPStreamer;

public class AudioRecorder extends Settings {
	public static Logger logger = Logger.getLogger(ScreencastApplet.class
			.getName());

	
	private AudioFormat audioFormat;
	private TargetDataLine targetDataLine;
	private Thread recorderThread;
	File audioFile = null;

	public void start() {
		logger.info("Entering start()");

		recorderThread.start();
		logger.info("Exiting start()");

	}

	public void stop() {
		logger.info("Entering stop()");
		targetDataLine.close();
		logger.info("line closed");
		logger.info("Streaming Audio File");
		new HTTPStreamer(audioFile, SessionFactory.getSession());
		logger.info("Deleting Audio file");
		audioFile.delete();
		logger.info("Exiting stop()");

	}

	public AudioRecorder() {
		logger.info("Entering AudioRecorder()");
		recorderThread = new CaptureAudioThread();
		audioFormat = getAudioFormat();
		DataLine.Info dataLineInfo = null;

		try {
			dataLineInfo = new DataLine.Info(TargetDataLine.class, audioFormat);
			logger.info("Little Endian machine detected");
		} catch (IllegalArgumentException e) {
			logger.info("Big Endian machine detected");
			dataLineInfo = new DataLine.Info(TargetDataLine.class,
					getBigEndianAudioFormat());
		}

		try {
			targetDataLine = (TargetDataLine) AudioSystem.getLine(dataLineInfo);
		} catch (LineUnavailableException e) {
			
			logger.info(e.getMessage());
		}
		logger.info("Exiting AudioRecorder()");

	}// end constructor

	private AudioFormat getAudioFormat() {

		return new AudioFormat(SAMPLE_RATE, SAMPLE_SIZE_IN_BITS, CHANNELS,
				SIGNED, BIG_ENDIAN);
	}

	private AudioFormat getBigEndianAudioFormat() {
		return new AudioFormat(SAMPLE_RATE, SAMPLE_SIZE_IN_BITS, CHANNELS,
				SIGNED, true);
	}

	class CaptureAudioThread extends Thread {
		public void run() {
			AudioFileFormat.Type fileType = null;

			fileType = AudioFileFormat.Type.WAVE;
			audioFile = new File(AUDIO_FILE_NAME);
			try {
				audioFile.createNewFile();

			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			try {
				targetDataLine.open(audioFormat);
			} catch (LineUnavailableException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			targetDataLine.start();
			try {
				AudioSystem.write(new AudioInputStream(targetDataLine),
						fileType, audioFile);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
	}

}
