/*
Copyright 2008 York University

Licensed under the Educational Community License (ECL), Version 2.0 or the New
BSD license. You may not use this file except in compliance with one these
Licenses.

You may obtain a copy of the ECL 2.0 License and BSD License at
https://source.fluidproject.org/svn/LICENSE.txt

 */
package org.fluidproject.vulab.rascal.core.utils;

import java.io.File;

import org.fluidproject.vulab.rascal.core.SessionFactory;
import org.fluidproject.vulab.rascal.utils.http.HTTPStreamer;

public class FileStreamer implements Runnable {

	private Thread thread;
	private File dir;

	public FileStreamer(File dirToStreamFrom) {
		thread = new Thread(this);
		dir = dirToStreamFrom;
		thread.start();
	}

	public void finalize() {
		// stream anything left over and clean up
		thread.interrupt();
		streamFiles();
	}

	public void run() {
		try {
			while (true) {
				streamFiles();
				synchronized (this) {
					Thread.sleep(1);
				}

			}
		} catch (InterruptedException ie) {
			// stop running
		}

	}

	private synchronized void streamFiles() {

		File[] files = dir.listFiles();
		if (files != null) {

			for (int i = 0; i < files.length; i++) {
				new HTTPStreamer(files[i], SessionFactory.getSession());
				files[i].delete();
			}
		}

	}

}
