/*
Copyright 2008 York University

Licensed under the Educational Community License (ECL), Version 2.0 or the New
BSD license. You may not use this file except in compliance with one these
Licenses.

You may obtain a copy of the ECL 2.0 License and BSD License at
https://source.fluidproject.org/svn/LICENSE.txt

 */
package org.fluidproject.vulab.rascal.core.utils;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;

public class Frame {
	private BufferedImage image;
	private long timestamp;

	public Frame(BufferedImage image, long timestamp) {
		this.image = image;
		this.timestamp = timestamp;
	}

	public BufferedImage getImage() {
		return image;
	}

	public void setImage(BufferedImage image) {
		this.image = image;
	}

	public long getTimestamp() {
		return timestamp;
	}

	public String toString() {
		return "Frame at timestap: " + timestamp;
	}

	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}

	public void toFile(File dir) {
		File file = new File(dir.getName() + "/" + timestamp + ".jpg");
		try {
			ImageIO.write(image, "jpg", file);

		} catch (IOException e) {

		}
	}

	public void adjustImage() {
		acceleratedImage();
		image = scale(image, 0.5f);
		image = compress(image, 0.5f);
	}

	private void acceleratedImage() {
		BufferedImage bi = new BufferedImage(image.getWidth(), image
				.getHeight(), image.getType());
		Graphics g = bi.getGraphics();
		g.drawImage(image, 0, 0, null);
		image = bi;
	}

	private static BufferedImage scale(BufferedImage image, float scale) {
		Date now = new Date();
		int width = (int) (image.getWidth() * scale);
		int height = (int) (image.getHeight() * scale);
		Image img = image.getScaledInstance(width, height, Image.SCALE_SMOOTH);
		image.getGraphics().dispose();
		BufferedImage result = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_RGB);
		result.createGraphics().drawImage(img, 0, 0, null);
		img = null;
		image = null;
		Date now2 = new Date();
		return result;

	}

	private static BufferedImage compress(BufferedImage image, float quality) {
		try {

			Date now = new Date();
			Iterator<ImageWriter> writers = ImageIO
					.getImageWritersBySuffix("jpeg");
			ImageWriter writer = writers.next();
			ImageWriteParam param = writer.getDefaultWriteParam();
			param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
			param.setCompressionQuality(quality);
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			writer.setOutput(ImageIO.createImageOutputStream(out));
			writer.write(null, new IIOImage(image, null, null), param);
			byte[] data = out.toByteArray();
			ByteArrayInputStream in = new ByteArrayInputStream(data);

			out.close();
			BufferedImage result = ImageIO.read(in);
			in.close();
			writer = null;
			writers = null;
			param = null;
			Date now2 = new Date();

			return result;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

}
